/*
 * Structure definition for NeXus muon file information
 */


struct NXM_MUONUIF
{
	NXname		uif_element_name;
	NXname		uif_element_value;
};


struct NXM_MUONIDF
{
	/* NXrun */
	NXname		run_program_name;
	NXname		run_program_version;
	int			run_IDF_version;
	int			run_number;
	NXname		run_title;
	NXname		run_notes;
	NXname		run_analysis;
	NXname		run_lab;
	NXname		run_beamline;
	NXname		run_start_time;
	NXname		run_stop_time;
	NXname		run_duration;
	int			run_switching_states;
	/* NXuser */
	NXname		user_name;
	NXname		user_experiment_number;
	int			user_uif_array_length;
	struct NXM_MUONUIF *	user_uif_array;
	/* NXsample */
	NXname		sample_name;
	float		sample_temperature;
	NXname		sample_temperature_units;
	float		sample_magnetic_field;
	NXname		sample_magnetic_field_units;
	NXname		sample_shape;
	NXname		sample_magnetic_field_state;
	int			sample_magnetic_field_vector_available;
	float *		sample_magnetic_field_vector;
	NXname		sample_magnetic_field_vector_coord;
	NXname		sample_environment;
	/* NXinstrument */
	NXname		instrument_name;
	/* NXdetector */
	int			detector_number;
	NXname		detector_orientation;
	int			detector_angles_available;
	float *		detector_angles;
	NXname		detector_angles_coord;
	int			detector_deadtimes_available;
	float *		detector_deadtimes;
	NXname		detector_deadtimes_units;
	/* NXcollimator */
	NXname		collimator_type;
	NXname		collimator_aperture;
	/* NXbeam */
	float		beam_total_counts;
	NXname		beam_total_counts_units;
	int			beam_daereads;
	int			beam_frames;
	/* NXdata histogram_data_1 */
	int *		histogram_counts;
	int			histogram_number;
	int			histogram_length;
	int			histogram_t0_bin;
	int			histogram_first_good_bin;
	int			histogram_last_good_bin;
	float			histogram_resolution;
	NXname		histogram_resolution_units;
	float *		histogram_raw_time;
	NXname		histogram_raw_time_units;
	float *		histogram_corrected_time;
	NXname		histogram_corrected_time_units;
	int			histogram_grouping_available;
	int *		histogram_grouping;
	int			histogram_alpha_available;
	float *		histogram_alpha;
	/* NXlog temperature_log_1 */
	int						temperature_log_available;
	NXname					temperature_log_name;
	float *					temperature_log_values;
	NXname					temperature_log_values_units;
	float *					temperature_log_time;
	NXname					temperature_log_time_units;
	/* NXlog events_log_1 */
	int						events_log_available;
	NXname					events_log_name;
	float *					events_log_values;
	NXname					events_log_values_units;
	float *					events_log_time;
	NXname					events_log_time_units;
};


/*
 * Function definitions for reading NeXus muon file information
 */

int NXMread(char *nxfname, struct NXM_MUONIDF **nxfdata);
void NXMfreememory(struct NXM_MUONIDF *nxfdata);
